﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ComplexEventProcessing;
using Microsoft.ComplexEventProcessing.Linq;
using System.Diagnostics;
using DispatchDemo.InputAdapter1;
using DispatchDemo.OutputAdapter1;
using System.Threading;
using Microsoft.Win32;
using Microsoft.ComplexEventProcessing.ManagementService;
using Microsoft.ComplexEventProcessing.Diagnostics;
using Microsoft.ComplexEventProcessing.Adapters;
using System.ServiceModel;
using Microsoft.ComplexEventProcessing.Extensibility;
using System.Runtime.Serialization;

namespace DispatchDemo
{
    public class Program
    {
        static void Main(string[] args)
        {

            ServiceHost _dvHost = null;
            IManagementService _service = null;
            Server server = Server.Create("Default");
            IManagementService proxy = server.CreateManagementService();
            _service = proxy;
            _dvHost = new ServiceHost(proxy);

            WSHttpBinding binding = new WSHttpBinding(SecurityMode.Message);
            binding.Security.Message.ClientCredentialType = MessageCredentialType.Windows;
            binding.MaxReceivedMessageSize = 4655360;
            _dvHost.AddServiceEndpoint(typeof(IManagementService), binding, "http://localhost:8080");
            _dvHost.Open();

            Application application = server.CreateApplication("Dispatcher");
            TraceListener tracer = new ConsoleTraceListener();
            //Console.WriteLine(TrainLine.LineA.ToString());
            DispatcherInputConfig commonConfig = new DispatcherInputConfig
            {
                StopName = "Stop1",
                CrossLines = new TrainLine[] {TrainLine.LineA,TrainLine.LineB, TrainLine.LineC },
                StopCapacity = 6
            };

            var outputConfig = new DispatcherOutputConfig();
            var inputAdapter = application.CreateInputAdapter<DispatcherInputFactory>("TraininfoInput", "Description");
            var outputAdapter = application.CreateOutputAdapter<DispatcherOutputFactory>("TraininfoOutput","Description");
            filterExample(application, commonConfig, outputConfig, inputAdapter, outputAdapter);
            DiagnosticSettings settings = server.GetDiagnosticSettings(new Uri("cep:/Server/Application/Dispatcher/Query/filterExampleQuery")); 
            settings.Aspects |= DiagnosticAspect.PerformanceCounters;
            server.SetDiagnosticSettings(new Uri("cep:/Server/Application/Dispatcher/Query/filterExampleQuery"), settings); 

            foreach (var query in application.Queries.Values)
            {
                Console.WriteLine("\r\nQuery: " + query.Name);
                query.Start();
                DiagnosticView diagnosticView;
                do
                {
                    Thread.Sleep(100);
                    diagnosticView = query.Application.Server.GetDiagnosticView(query.Name);

                } while ((string)diagnosticView["QueryState"] == "Running");
                query.Stop();
            }


            application.Delete();
            server.Dispose();
            Console.Read();

        }

        /// <summary>
        /// Binds a stream to output adapter and runs the query
        /// </summary>
        /// <param name="cepStream"></param>
        private static void runQuery(CepStream<TrainRuntimeInfo> cepStream, Application application)
        {
            // Configure output adapter
            var outputConfig = new DispatcherOutputConfig();

            // Create query and bind to the output adapter
            var query = cepStream.ToQuery(application, Guid.NewGuid().ToString(), "description", typeof(DispatcherOutputFactory), outputConfig, EventShape.Point, StreamEventOrder.ChainOrdered);

            // Start query
            query.Start();

            // Wait until query change state
            DiagnosticView diagnosticView;
            do
            {
                Thread.Sleep(100);
                diagnosticView = query.Application.Server.GetDiagnosticView(query.Name);
            } while ((string)diagnosticView[DiagnosticViewProperty.QueryState] == "Running");

            // Stop query
            query.Stop();
        }

        /// <summary>
        /// Example of filter function
        /// </summary>
        /// <param name="cepStream"></param>
        /// <param name="outputConfig"></param>
        /// <param name="adapterStopSignal"></param>
        private static Query filterExample(Application application, DispatcherInputConfig inputConfig, DispatcherOutputConfig outputConfig, InputAdapter inputAdapter, OutputAdapter outputAdapter)
        {
            var input = CepStream<TrainRuntimeInfo>.Create("input");
            var UDSOStream = input.Scan(new UDSOExample<TrainRuntimeInfo>(false));
            var filteredCepStream = from e in UDSOStream
                                    where e.TargetLine == "LineA"
                                    select e;

            var queryTemplate = application.CreateQueryTemplate("filterExampleTemplate", "Description...", filteredCepStream);

            var queryBinder = new QueryBinder(queryTemplate);
            queryBinder.BindProducer<TrainRuntimeInfo>("input", inputAdapter, inputConfig, EventShape.Point);
            queryBinder.AddConsumer<TrainRuntimeInfo>("output", outputAdapter, outputConfig, EventShape.Point, StreamEventOrder.ChainOrdered);

            var query = application.CreateQuery("filterExampleQuery", "Description...", queryBinder);
            return query;
        }

    }

    [DataContract]
    public class UDSOExample<T> : CepPointStreamOperator<T, T> where T : TrainRuntimeInfo
    {
        [DataMember]
        readonly bool _isEmpty;

        public UDSOExample(bool isEmpty)
        {
            _isEmpty = isEmpty;
        }

        public override IEnumerable<T> ProcessEvent(PointEvent<T> inputEvent)
        {
            if (inputEvent.EventKind == EventKind.Insert)
            {
                yield return inputEvent.Payload;
                T newPayload = inputEvent.Payload;
                newPayload.TargetLine = "MyNewLine";
                yield return newPayload;
            }
        }
        public override System.DateTimeOffset? NextCti
        {
            get { return base.NextCti; }
        }

        public override bool IsEmpty
        {
            get { return _isEmpty; }
        }
    }
}
